@@ -1,6 +1,9 @@
 from datetime import date
 
-from .utils import streams
+from .utils import (
+    len_color,
+    streams
+)
 from .shortcuts import progressbar
 
 from .widgets import (
@@ -20,6 +23,7 @@ from .widgets import (
     ReverseBar,
     BouncingBar,
     RotatingMarker,
+    Variable,
     DynamicMessage,
     FormatCustomText,
     CurrentTime
@@ -41,6 +45,7 @@ from .__about__ import (
 __date__ = str(date.today())
 __all__ = [
     'progressbar',
+    'len_color',
     'streams',
     'Timer',
     'ETA',
@@ -61,6 +66,7 @@ __all__ = [
     'ProgressBar',
     'DataTransferBar',
     'RotatingMarker',
+    'Variable',
     'DynamicMessage',
     'FormatCustomText',
     'CurrentTime',
